/*
 *  cfmglue.c
 *  MoCxConduit
 *
 *  Created by Daniel Braun on 12/03/05.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *  parts of code comes from MachOCFMGlue.c Copyright (c) 2002 by M. Uli Kusterer
 *
 * note : this is really C, but syncmgr.h requires c++ 
 */
#include <unistd.h>

#include <CodeFragments.h>
#include "SYNCMGR.h"
#include "UserMgr.h"
#include "cfmglue.h"
#include "hslog.h"

extern "C" {
	long LogAddEntry(LPCTSTR inEntry, Activity act, PALMBOOL bTimeStamp);
}

static void*	MachOFunctionPointerForCFMFunctionPointer( void* inCfmProcPtr );
static void	DisposeMachOFunctionPointer( void* inMachProcPtr );



typedef struct {
	const unsigned char *symbol;
	void *symAddr;
	void *wrapper;
} funcentry_t;


#define WRAP(func, returntype, errval, args_with_type, args)		\
returntype func args_with_type						\
{									\
	typedef returntype (*func##_t)args_with_type;			\
		func##_t f=(func##_t)FuncEntry[__##func].wrapper;	\
			if (!f) return errval;				\
				return f args;				\
}

/* WRAP macro expands, eg. to : 
long LogAddEntry(LPCTSTR pszEntry, Activity act,  BOOL bTimeStamp)
{
	typedef long (*LogAddEntry_t)(LPCTSTR pszEntry, Activity act,  PALMBOOL bTimeStamp); 
	LogAddEntry_t f=FuncEntry[__LogAddEntry].wrapper;
	if (!f) return -1;
	return f(pszEntry,act,bTimeStamp);
}

be carrefull with __xxx defines, the number must match the index in FuncEntry
*/

static funcentry_t FuncEntry[]={
#define __SyncRegisterConduit	0
	{"\pSyncRegisterConduit", NULL, NULL},
#define __LogAddEntry		1	
	{"\pLogAddEntry", NULL,NULL},
#define __SyncUnRegisterConduit 2
	{"\pSyncUnRegisterConduit", NULL, NULL},
#define	__SyncCallRemoteModule	3
	{"\pSyncCallRemoteModule", NULL, NULL},
#define __UmGetUserTempSyncPreferences 4
	{"\pUmGetUserTempSyncPreferences", NULL, NULL},
	/* more to be added */
#define __UmSetUserTempSyncPreferences 5
	{"\pUmSetUserTempSyncPreferences", NULL, NULL},
#define	__UmSetUserPermSyncPreferences 6
	{"\pUmSetUserPermSyncPreferences", NULL, NULL},
#define	__UmGetUserPermSyncPreferences 7
	{"\pUmSetUserPermSyncPreferences", NULL, NULL},

	{NULL, NULL, NULL}
};



WRAP(SyncUnRegisterConduit,	long, -42, (CONDHANDLE h), (h) )
WRAP(SyncRegisterConduit,	long, -42, (CONDHANDLE &ph), (ph) )
WRAP(SyncCallRemoteModule,	long, -42, (CCallModuleParams* pParams), (pParams) )
WRAP(LogAddEntry,		long, -42, (LPCTSTR pszEntry, Activity act,  PALMBOOL bTimeStamp), (pszEntry,act,bTimeStamp) )
WRAP(UmGetUserTempSyncPreferences, OSStatus, -42,
     (PalmUserID iUserID, ConduitCreator iConduitCreator, UmUserSyncAction *oSyncPrefs),
     (iUserID, iConduitCreator, oSyncPrefs))
WRAP(UmGetUserPermSyncPreferences, OSStatus, -42,
     (PalmUserID iUserID, ConduitCreator iConduitCreator, UmUserSyncAction *oSyncPrefs),
     (iUserID, iConduitCreator, oSyncPrefs))
WRAP(UmSetUserTempSyncPreferences, OSStatus, -42,
     (PalmUserID iUserID, ConduitCreator iConduitCreator, UmUserSyncAction iSyncPrefs),
     ( iUserID,  iConduitCreator,  iSyncPrefs))
WRAP(UmSetUserPermSyncPreferences, OSStatus, -42,
     (PalmUserID iUserID, ConduitCreator iConduitCreator, UmUserSyncAction iSyncPrefs),
     ( iUserID,  iConduitCreator,  iSyncPrefs))

/* more to be added */

static CFragConnectionID connID=0;



int  CfmGmlueGetHotSyncLib(void)
{
	OSErr err;
	Ptr mainAddr;
	Str255 errMsg="";
	int i;	
	
	if (connID) return -1; // already called ?
	FILE *F=fopen("/tmp/totoMap", "w"); setlinebuf(F);

	err=GetSharedLibrary ("\pHotSyncLib.PPC",
			      'pwpc',
			      1, 
			      &connID,
			      &mainAddr,
			      errMsg);
	if (err) {
		fprintf(F, "GetSharedLibrary err %d errmsg %s\n", err, errMsg);
		return err;
	}
	for (i=0; ;i++) {
		CFragSymbolClass symClass;
		if (!FuncEntry[i].symbol) break;
		fprintf(F, "mapping %s\n", FuncEntry[i].symbol+1);
		FindSymbol(connID, FuncEntry[i].symbol, (Ptr *)&(FuncEntry[i].symAddr), &symClass);
		fprintf(F, "	mapped %s => %X\n", FuncEntry[i].symbol+1, FuncEntry[i].symAddr);
		FuncEntry[i].wrapper=MachOFunctionPointerForCFMFunctionPointer(FuncEntry[i].symAddr);
		fprintf(F, "	wrapp %s => %X\n", FuncEntry[i].symbol+1, FuncEntry[i].wrapper);
	}
	return 0;	
}

void CfmGlueFreeHotSyncLib(void)
{
	int i;	
	if (!connID) return; // not loaded or already free
	for (i=0; ;i++) {
		if (!FuncEntry[i].symbol) break;
		if (!FuncEntry[i].wrapper) continue;
		DisposeMachOFunctionPointer(FuncEntry[i].wrapper);
	}
	CloseConnection(&connID);
	connID=0;
}




/* =============================================================================
PROJECT:	Cubix (PB1)
FILE:		MachOCFMGlue.c

PURPOSE:	Code that does the magic needed to call MachO functions
from CFM and CFM functions from Carbon.

COPYRIGHT:	Copyright (c) 2002 by M. Uli Kusterer, all rights reserved.
Thanks to George Warner, Chris Silverberg and Ricky Sharp for
clues on how to do this and implementation snippets.

REVISIONS:
Fri Jul 26 2002	witness	Created.
========================================================================== */
typedef struct TVector_struct
{
	ProcPtr fProcPtr;
	UInt32 fTOC;
} TVector_rec, *TVector_ptr;


static UInt32 gGlueTemplate[6] = { 0x3D800000, 0x618C0000, 0x800C0000, 0x804C0004,
	0x7C0903A6, 0x4E800420};



/* -----------------------------------------------------------------------------
MachOFunctionPointerForCFMFunctionPointer:
This function allocates a block of CFM glue code which contains the
instructions to call a CFM routine from a MachO application.

Syntax analogous to CFMFunctionPointerForMachOFunctionPointer().

REVISIONS:
Fri Jul 26 2002	witness	Created.
-------------------------------------------------------------------------- */

static void*	MachOFunctionPointerForCFMFunctionPointer( void* inCfmProcPtr )
{
	UInt32	*vMachProcPtr = (UInt32*) NewPtr( sizeof(gGlueTemplate) );	// sizeof() really returns the data size here, not the size of a pointer. Trust me.
	
	vMachProcPtr[0] = gGlueTemplate[0] | ((UInt32)inCfmProcPtr >> 16);
	vMachProcPtr[1] = gGlueTemplate[1] | ((UInt32)inCfmProcPtr & 0xFFFF);
	vMachProcPtr[2] = gGlueTemplate[2];
	vMachProcPtr[3] = gGlueTemplate[3];
	vMachProcPtr[4] = gGlueTemplate[4];
	vMachProcPtr[5] = gGlueTemplate[5];
	MakeDataExecutable( vMachProcPtr, sizeof(gGlueTemplate) );
	
	return( vMachProcPtr );
}


/* -----------------------------------------------------------------------------
DisposeMachOFunctionPointer:
Disposes of the fake TVector created by
CFMFunctionPointerForMachOFunctionPointer().

REVISIONS:
Fri Jul 26 2002	witness	Created.
-------------------------------------------------------------------------- */

static void DisposeMachOFunctionPointer( void *inMachProcPtr )
{
	if( inMachProcPtr )
		DisposePtr((char *) inMachProcPtr );
}
/* =============================================================================
end of MachOCFMGlue code
=============================================================================
*/
