/*
 *  syncproxy.h
 *  MoCxConduit
 *
 *  Created by Daniel Braun on 13/03/05.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef SYNC_PROXY_H
#define SYNC_PROXY_H

// we use SOCK_STREAM sockets, since SOCK_SEQPACKET is not supported
// on PF_UNIX.
// may be one day one would discover that telecom is about packets, not
// stream of bytes.....

// overall packet is : 
//	unsigned long packet_len  (whole packet length in bytes, not counting this field)
//	SYNCBASE		  (common fields to all packets)
//	specific part

// hotsync_send / hotsync_recv helpers to send/recv such packets.
// no real PAD here, we assume all data available, this is ok
// only on unix domain, not on inet domain


static inline int hotsync_send(int soc, char *data, unsigned long len)	
{
	int rc;
	write(soc, &len, sizeof(len));
	rc=write(soc, data, len);
#ifdef _LOGSYNC
#ifdef _IN_PROXY
	{ 
		char szMsg[256];
		sprintf(szMsg,"hotsync_send len %lu rc=%d at %s:%d / errno %d",
			len, rc, __FILE__, __LINE__, errno);
		LogAddEntry(szMsg, slText, 1);
	}
#else
	NSLog(@"hotsync_send len %d rc=%d at %s:%d errno %d\n", len, rc, __FILE__, __LINE__, errno);
#endif
#endif
	return rc;
}
static inline int hotsync_recv(int soc, char *buffer, unsigned long maxlen)
{
	unsigned long len;
	int rc;
	rc=read(soc, &len, sizeof(len));
#ifdef _LOGSYNC
#ifdef _IN_PROXY
	if (rc<=0) { 
		char szMsg[256];
		sprintf(szMsg,"hotsync_recv1  rc=%d at %s:%d errno %d",
			 rc, __FILE__, __LINE__,errno);
		LogAddEntry(szMsg, slText, 1);
	}
#else
	if (rc<=0) NSLog(@"hotsync_recv1 rc=%d at %s:%d errno %d\n",  rc, __FILE__, __LINE__, errno);
#endif
#endif
	if (rc<=0) return rc;
	if (len>maxlen) return -2;
	rc=read(soc, buffer, len);
#ifdef _LOGSYNC
#ifdef _IN_PROXY
	{ 
		char szMsg[256];
		sprintf(szMsg,"hotsync_recv len %lu rc=%d at %s:%d",
			len, rc, __FILE__, __LINE__);
		LogAddEntry(szMsg, slText, 1);
	}
#else
	NSLog(@"hotsync_recv len %d rc=%d at %s:%d errno %d\n", len, rc, __FILE__, __LINE__, errno);
#endif
#endif
	return rc;
}
	

#define HOTSYNC_
#define SYNCBASE \
	unsigned char code; \
	unsigned char flags; \
	unsigned long tid; \
	unsigned int  reserved1; \
	unsigned long reserved2; \



struct syncproxy_any {
	SYNCBASE
};

#define SYNC_PROXY_REMOTEAPP		0x01

struct syncproxy_callparam {
	SYNCBASE
	unsigned long creatorid;
	unsigned long typid;
	unsigned long actioncode;
	unsigned char data[0];
};

#define SYNC_PROXY_RESULT		0x02
struct syncproxy_result {
	SYNCBASE
	unsigned long rc; // rc from hotsync api
	unsigned long rc2; // rc from palm app for rpc
	unsigned char data[0];
};

#define SYNC_PROXY_VERSION		0x03
struct syncproxy_version {
	SYNCBASE
	unsigned long version;
	unsigned long minproxyversion;
};

#define SYNC_PROXY_VERSION_VALUE 0

#define SYNC_PROXY_END		0x04


#endif
